@@ -311,7 +311,7 @@ class ModifiedCELOERefinement(BaseRefinement[OENode]):
         for i in refs:
             # assert i is not None
             # yield i
-            i_inds = self.kb.individuals_set(i)
+            i_inds = None
             for j in refs:
                 # assert j is not None
                 if (i, j) in mem or i == j:
@@ -322,6 +322,8 @@ class ModifiedCELOERefinement(BaseRefinement[OENode]):
 
                 if (max_length >= length) and (self.max_child_length >= length + 1):
                     if not i.is_owl_thing() and not j.is_owl_thing():
+                        if i_inds is None:
+                            i_inds = self.kb.individuals_set(i)
                         j_inds = self.kb.individuals_set(j)
                         if not j_inds.difference(i_inds):
                             # already contained
@@ -596,121 +598,6 @@ class CustomRefinementOperator(BaseRefinement[Node]):
             raise ValueError
 
 
-class ExampleRefinement(BaseRefinement):
-    """
-     A top down/downward refinement operator refinement operator in ALC.
-    """
-
-    def __init__(self, knowledge_base: KnowledgeBase):
-        super().__init__(knowledge_base)
-
-    @parametrized_performance_debugger()
-    def refine_atomic_concept(self, concept: 'Concept') -> Set:
-        """
-        # (1) Create all direct sub concepts of C that are defined in TBOX.
-        # (2) Create negations of all leaf concepts in  the concept hierarchy.
-        # (3) Create ∀.r.T and ∃.r.T where r is the most general relation.
-        # (4) Intersect and union set of concepts that are generated in (1-3).
-        # Note that this is modified implementation of refinemenet operator proposed in
-        Concept Learning in Description Logics Using Refinement Operators
-
-        :param concept: Concept
-        :return: A set of refinements.
-        """
-        # (1) Generate all direct_sub_concepts
-        sub_concepts = self.kb.get_direct_sub_concepts(concept)
-        # (2) Create negation of all leaf_concepts
-        negs = self.kb.negation_from_iterables(self.kb.get_leaf_concepts(concept))
-        # (3) Create ∃.r.T where r is the most general relation.
-        existential_rest = self.kb.most_general_existential_restrictions(concept)
-        universal_rest = self.kb.most_general_universal_restriction(concept)
-        a, b = tee(chain(sub_concepts, negs, existential_rest, universal_rest))
-
-        mem = set()
-        for i in a:
-            yield i
-            for j in copy.copy(b):
-                if (i == j) or ((i.str, j.str) in mem) or ((j.str, i.str) in mem):
-                    continue
-                mem.add((j.str, i.str))
-                mem.add((i.str, j.str))
-                mem.add((j.str, i.str))
-                yield self.kb.union(i, j)
-                yield self.kb.intersection(i, j)
-
-    def refine_complement_of(self, concept: 'Concept'):
-        """
-        :type concept: Concept
-        :param concept:
-        :return:
-        """
-        parents = self.kb.get_direct_parents(self.kb.negation(concept))
-        return self.kb.negation_from_iterables(parents)
-
-    def refine_object_some_values_from(self, concept: 'Concept'):
-        for i in self.refine(concept.filler):
-            yield self.kb.existential_restriction(i, concept.role)
-
-    def refine_object_all_values_from(self, C: 'Concept'):
-        """
-
-        :param C:
-        :return:
-        """
-        # rule 1: Forall r.D = > Forall r.E
-        for i in self.refine(C.filler):
-            yield self.kb.universal_restriction(i, C.role)
-
-    def refine_object_union_of(self, C: 'Concept'):
-        """
-
-        :param C:
-        :return:
-        """
-        concept_A = C.concept_a
-        concept_B = C.concept_b
-        for ref_concept_A in self.refine(concept_A):
-            yield self.kb.union(ref_concept_A, concept_B)
-
-        for ref_concept_B in self.refine(concept_B):
-            yield self.kb.union(ref_concept_B, concept_A)
-
-    def refine_object_intersection_of(self, C: 'Concept'):
-        """
-
-        :param C:
-        :return:
-        """
-
-        result = set()
-        concept_A = C.concept_a
-        concept_B = C.concept_b
-        for ref_concept_A in self.refine(concept_A):
-            result.add(self.kb.intersection(ref_concept_A, concept_B))
-
-        for ref_concept_A in self.refine(concept_A):
-            result.add(self.kb.intersection(ref_concept_A, concept_B))
-
-        return result
-
-    def refine(self, ce: OWLClassExpression) -> Iterable[OWLClassExpression]:
-        assert isinstance(ce, OWLClassExpression)
-        if isinstance(ce, OWLClass):
-            yield from self.refine_atomic_concept(ce)
-        elif isinstance(ce, OWLObjectComplementOf):
-            yield from self.refine_complement_of(ce)
-        elif isinstance(ce, OWLObjectSomeValuesFrom):
-            yield from self.refine_object_some_values_from(ce)
-        elif isinstance(ce, OWLObjectAllValuesFrom):
-            yield from self.refine_object_all_values_from(ce)
-        elif isinstance(ce, OWLObjectUnionOf):
-            yield from self.refine_object_union_of(ce)
-        elif isinstance(ce, OWLObjectIntersectionOf):
-            yield from self.refine_object_intersection_of(ce)
-        else:
-            raise ValueError
-
-
 class ExpressRefinement(BaseRefinement[Node]):
     """ A top down refinement operator refinement operator in ALC."""
 
