@@ -4,16 +4,30 @@ from logging import warning
 from typing import Iterable, Set, Final, cast
 
 import owlready2
+from pandas import Timedelta
+from owlready2 import declare_datatype
 
 from owlapy import namespaces
 from owlapy.model import OWLOntologyManager, OWLOntology, OWLClass, OWLDataProperty, OWLObjectProperty, \
     OWLNamedIndividual, OWLReasoner, OWLClassExpression, OWLObjectPropertyExpression, OWLOntologyID, OWLAxiom, \
     OWLOntologyChange, AddImport, OWLEquivalentClassesAxiom, OWLThing, OWLAnnotationAssertionAxiom, DoubleOWLDatatype, \
-    IRI, OWLObjectInverseOf \
+    IRI, OWLObjectInverseOf, OWLLiteral \
     # OWLObjectSomeValuesFrom, OWLProperty, \
 from owlapy.owlready2.utils import ToOwlready2
 
 
+def _parse_duration_datatype(literal: str):
+    return Timedelta(literal)
+
+
+def _unparse_duration_datatype(literal: Timedelta):
+    return literal.isoformat()
+
+
+declare_datatype(Timedelta, "http://www.w3.org/2001/XMLSchema#duration",
+                 _parse_duration_datatype, _unparse_duration_datatype)
+
+
 class BaseReasoner_Owlready2(Enum):
     PELLET = auto()
 
@@ -23,8 +37,11 @@ class OWLOntologyManager_Owlready2(OWLOntologyManager):
 
     _world: owlready2.namespace.World
 
-    def __init__(self):
-        self._world = owlready2.World()
+    def __init__(self, world_store=None):
+        if world_store is None:
+            self._world = owlready2.World()
+        else:
+            self._world = owlready2.World(filename=world_store)
 
     def create_ontology(self, iri: IRI) -> 'OWLOntology_Owlready2':
         return OWLOntology_Owlready2(self, iri, load=False)
@@ -100,6 +117,9 @@ class OWLOntologyManager_Owlready2(OWLOntologyManager):
             # TODO XXX
             raise NotImplementedError
 
+    def save_world(self):
+        self._world.save()
+
 
 _VERSION_IRI: Final = IRI.create(namespaces.OWL, "versionIRI")
 
@@ -217,7 +237,7 @@ class OWLReasoner_Owlready2(OWLReasoner):
         i: owlready2.Thing = self._world[ind.get_iri().as_str()]
         p: owlready2.DataPropertyClass = self._world[pe.get_iri().as_str()]
         for val in p._get_values_for_individual(i):
-            yield val
+            yield OWLLiteral(val)
 
     def object_property_values(self, ind: OWLNamedIndividual, pe: OWLObjectPropertyExpression) \
             -> Iterable[OWLNamedIndividual]:
